@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# vim: ts=4 sw=4 et ai si
+# vim: ts=4 sw=4 tw=88 et ai si
 #
 # Copyright (c) 2012-2014 Intel, Inc.
 # License: GPLv2
@@ -57,6 +57,7 @@ also contribute to the mapped blocks and are also copied.
 # pylint: disable=R0915
 
 import os
+import re
 import stat
 import sys
 import hashlib
@@ -65,6 +66,7 @@ import datetime
 from six import reraise
 from six.moves import queue as Queue
 from six.moves import _thread as thread
+from typing import Optional
 from xml.etree import ElementTree
 from bmaptools.BmapHelpers import human_size
 
@@ -80,9 +82,90 @@ class Error(Exception):
     support only one type of exceptions, and we basically throw human-readable
     problem description in case of errors.
     """
+
     pass
 
 
+class SysfsChange:
+    """Context manager for temporary sysfs changes
+    Writes a temporary value to a sysfs file and restores the original value upon exit.
+    Does a read access first to see if the requested value is already set. In that case,
+    no write access is made to not cause unnecessary errors when running as a non-root
+    user.
+    Can deal with sysfs files that return their plain current value upon reading, as
+    well as those that return all possible values with the current one in square
+    brackets. In this case, all possible values are stored in the `options` attribute.
+    """
+
+    re_current_selection = re.compile(r"([^\[\]]*)\[([^ \]]+)\]([^\[\]]*)")
+
+    def __init__(self, path: str, temp_value: str, suppress_ioerrors=True) -> None:
+        """The class constructor. The parameters are:
+        path       - The sysfs file to change
+        temp_value - The value to set upon entering the context manager
+        suppress_ioerrors - When True, an IOError during __enter__ is stored to the
+                            `error` attribute. When False, the exception is raised
+                            directly.
+        """
+        self.path = path
+        self.temp_value = temp_value
+        self.suppress_ioerrors = suppress_ioerrors
+        self.old_value = ""
+        self.modified = False
+        self.options = []
+        self.error: Optional[IOError] = None
+
+    def _read(self):
+        with open(self.path, "r") as f:
+            contents = f.read().strip()
+
+        # Some sysfs files return a list of options with the current selection
+        # in square brackets, e.g. "[mq-deadline] none" for the I/O scheduler.
+        # Return only the current option in that case.
+        match = self.re_current_selection.match(contents)
+        if match:
+            self.options = "".join(match.groups()[0:3]).split(" ")
+            return match.group(2)
+        else:
+            return contents
+
+    def _write(self, value):
+        with open(self.path, "w") as f:
+            f.write(value)
+
+    def __enter__(self):
+        try:
+            self.old_value = self._read()
+            _log.debug(f"found {self.path} to be '{self.old_value}'")
+        except IOError as exc:
+            if self.suppress_ioerrors:
+                self.error = exc
+                return self
+            else:
+                raise
+
+        if self.old_value != self.temp_value:
+            try:
+                _log.debug(f"setting {self.path} to '{self.temp_value}'")
+                self._write(self.temp_value)
+                self.modified = True
+            except IOError as exc:
+                if self.suppress_ioerrors:
+                    self.error = exc
+                else:
+                    raise
+        return self
+
+    def __exit__(self, exc_type, exc_value, exc_tb):
+        if self.modified:
+            try:
+                _log.debug(f"setting {self.path} back to '{self.old_value}'")
+                self._write(self.old_value)
+            except IOError as exc:
+                raise Error(f"cannot restore {self.path} to '{self.old_value}': {exc}")
+        return False
+
+
 class BmapCopy(object):
     """
     This class implements the bmap-based copying functionality. To copy an
@@ -172,6 +255,7 @@ class BmapCopy(object):
         self._progress_file = None
         self._progress_format = None
         self.set_progress_indicator(None, None)
+        self._psplash_pipe = None
 
         self._f_image = image
         self._image_path = image.name
@@ -188,9 +272,11 @@ class BmapCopy(object):
         self._bmap_cs_attrib_name = None
 
         # Special quirk for /dev/null which does not support fsync()
-        if stat.S_ISCHR(st_data.st_mode) and \
-           os.major(st_data.st_rdev) == 1 and \
-           os.minor(st_data.st_rdev) == 3:
+        if (
+            stat.S_ISCHR(st_data.st_mode)
+            and os.major(st_data.st_rdev) == 1
+            and os.minor(st_data.st_rdev) == 3
+        ):
             self._dest_supports_fsync = False
         else:
             self._dest_supports_fsync = True
@@ -211,6 +297,29 @@ class BmapCopy(object):
 
         self._batch_blocks = self._batch_bytes // self.block_size
 
+    def set_psplash_pipe(self, path):
+        """
+        Set the psplash named pipe file path to be used when updating the
+        progress - best effort.
+
+        The 'path' argument is the named pipe used by the psplash process to get
+        progress bar commands. When the path argument doesn't exist or is not a
+        pipe, the function will ignore with a warning. This behavior is
+        considered as such because the progress is considered a decoration
+        functionality which might or might not be available even if requested.
+        When used as a boot service, the unavailability of the psplash service
+        (due to various reasons: no screen, racing issues etc.) should not
+        break the writting process. This is why this implementation is done as
+        a best effort.
+        """
+
+        if os.path.exists(path) and stat.S_ISFIFO(os.stat(path).st_mode):
+            self._psplash_pipe = path
+        else:
+            _log.warning(
+                "'%s' is not a pipe, so psplash progress will not be " "updated" % path
+            )
+
     def set_progress_indicator(self, file_obj, format_string):
         """
         Setup the progress indicator which shows how much data has been copied
@@ -236,9 +345,11 @@ class BmapCopy(object):
         """
 
         if self.image_size is not None and self.image_size != image_size:
-            raise Error("cannot set image size to %d bytes, it is known to "
-                        "be %d bytes (%s)" % (image_size, self.image_size,
-                                              self.image_size_human))
+            raise Error(
+                "cannot set image size to %d bytes, it is known to "
+                "be %d bytes (%s)"
+                % (image_size, self.image_size, self.image_size_human)
+            )
 
         self.image_size = image_size
         self.image_size_human = human_size(image_size)
@@ -261,13 +372,12 @@ class BmapCopy(object):
         # Before verifying the shecksum, we have to substitute the checksum
         # value stored in the file with all zeroes. For these purposes we
         # create private memory mapping of the bmap file.
-        mapped_bmap = mmap.mmap(self._f_bmap.fileno(), 0,
-                                access=mmap.ACCESS_COPY)
+        mapped_bmap = mmap.mmap(self._f_bmap.fileno(), 0, access=mmap.ACCESS_COPY)
 
         chksum_pos = mapped_bmap.find(correct_chksum.encode())
         assert chksum_pos != -1
 
-        mapped_bmap[chksum_pos:chksum_pos + self._cs_len] = b'0' * self._cs_len
+        mapped_bmap[chksum_pos : chksum_pos + self._cs_len] = b"0" * self._cs_len
 
         hash_obj = hashlib.new(self._cs_type)
         hash_obj.update(mapped_bmap)
@@ -276,9 +386,11 @@ class BmapCopy(object):
         mapped_bmap.close()
 
         if calculated_chksum != correct_chksum:
-            raise Error("checksum mismatch for bmap file '%s': calculated "
-                        "'%s', should be '%s'"
-                        % (self._bmap_path, calculated_chksum, correct_chksum))
+            raise Error(
+                "checksum mismatch for bmap file '%s': calculated "
+                "'%s', should be '%s'"
+                % (self._bmap_path, calculated_chksum, correct_chksum)
+            )
 
     def _parse_bmap(self):
         """
@@ -295,20 +407,24 @@ class BmapCopy(object):
                 if num >= err.position[0] - 4 and num <= err.position[0] + 4:
                     xml_extract += "Line %d: %s" % (num, line)
 
-            raise Error("cannot parse the bmap file '%s' which should be a "
-                        "proper XML file: %s, the XML extract:\n%s" %
-                        (self._bmap_path, err, xml_extract))
+            raise Error(
+                "cannot parse the bmap file '%s' which should be a "
+                "proper XML file: %s, the XML extract:\n%s"
+                % (self._bmap_path, err, xml_extract)
+            )
 
         xml = self._xml
-        self.bmap_version = str(xml.getroot().attrib.get('version'))
+        self.bmap_version = str(xml.getroot().attrib.get("version"))
 
         # Make sure we support this version
-        self.bmap_version_major = int(self.bmap_version.split('.', 1)[0])
-        self.bmap_version_minor = int(self.bmap_version.split('.', 1)[1])
-        if self.bmap_version_major > int(SUPPORTED_BMAP_VERSION.split('.', 1)[0]):
-            raise Error("only bmap format version up to %d is supported, "
-                        "version %d is not supported"
-                        % (SUPPORTED_BMAP_VERSION, self.bmap_version_major))
+        self.bmap_version_major = int(self.bmap_version.split(".", 1)[0])
+        self.bmap_version_minor = int(self.bmap_version.split(".", 1)[1])
+        if self.bmap_version_major > int(SUPPORTED_BMAP_VERSION.split(".", 1)[0]):
+            raise Error(
+                "only bmap format version up to %d is supported, "
+                "version %d is not supported"
+                % (SUPPORTED_BMAP_VERSION, self.bmap_version_major)
+            )
 
         # Fetch interesting data from the bmap XML file
         self.block_size = int(xml.find("BlockSize").text.strip())
@@ -322,12 +438,15 @@ class BmapCopy(object):
 
         blocks_cnt = (self.image_size + self.block_size - 1) // self.block_size
         if self.blocks_cnt != blocks_cnt:
-            raise Error("Inconsistent bmap - image size does not match "
-                        "blocks count (%d bytes != %d blocks * %d bytes)"
-                        % (self.image_size, self.blocks_cnt, self.block_size))
+            raise Error(
+                "Inconsistent bmap - image size does not match "
+                "blocks count (%d bytes != %d blocks * %d bytes)"
+                % (self.image_size, self.blocks_cnt, self.block_size)
+            )
 
-        if self.bmap_version_major > 1 or \
-           (self.bmap_version_major == 1 and self.bmap_version_minor == 4):
+        if self.bmap_version_major > 1 or (
+            self.bmap_version_major == 1 and self.bmap_version_minor == 4
+        ):
             # In bmap format version 1.0-1.3 the only supported checksum type
             # was SHA1. Version 2.0 started supporting arbitrary checksum
             # types. A new "ChecksumType" tag was introduce to specify the
@@ -354,8 +473,9 @@ class BmapCopy(object):
             try:
                 self._cs_len = len(hashlib.new(self._cs_type).hexdigest())
             except ValueError as err:
-                raise Error("cannot initialize hash function \"%s\": %s" %
-                            (self._cs_type, err))
+                raise Error(
+                    'cannot initialize hash function "%s": %s' % (self._cs_type, err)
+                )
             self._verify_bmap_checksum()
 
     def _update_progress(self, blocks_written):
@@ -368,41 +488,52 @@ class BmapCopy(object):
         if self.mapped_cnt:
             assert blocks_written <= self.mapped_cnt
             percent = int((float(blocks_written) / self.mapped_cnt) * 100)
-            _log.debug("wrote %d blocks out of %d (%d%%)" %
-                       (blocks_written, self.mapped_cnt, percent))
+            _log.debug(
+                "wrote %d blocks out of %d (%d%%)"
+                % (blocks_written, self.mapped_cnt, percent)
+            )
         else:
             _log.debug("wrote %d blocks" % blocks_written)
 
-        if not self._progress_file:
-            return
+        if self._progress_file:
+            if self.mapped_cnt:
+                progress = "\r" + self._progress_format % percent + "\n"
+            else:
+                # Do not rotate the wheel too fast
+                now = datetime.datetime.now()
+                min_delta = datetime.timedelta(milliseconds=250)
+                if now - self._progress_time < min_delta:
+                    return
+                self._progress_time = now
 
-        if self.mapped_cnt:
-            progress = '\r' + self._progress_format % percent + '\n'
-        else:
-            # Do not rotate the wheel too fast
-            now = datetime.datetime.now()
-            min_delta = datetime.timedelta(milliseconds=250)
-            if now - self._progress_time < min_delta:
-                return
-            self._progress_time = now
+                progress_wheel = ("-", "\\", "|", "/")
+                progress = "\r" + progress_wheel[self._progress_index % 4] + "\n"
+                self._progress_index += 1
 
-            progress_wheel = ('-', '\\', '|', '/')
-            progress = '\r' + progress_wheel[self._progress_index % 4] + '\n'
-            self._progress_index += 1
+            # This is a little trick we do in order to make sure that the next
+            # message will always start from a new line - we switch to the new
+            # line after each progress update and move the cursor up. As an
+            # example, this is useful when the copying is interrupted by an
+            # exception - the error message will start form new line.
+            if self._progress_started:
+                # The "move cursor up" escape sequence
+                self._progress_file.write("\033[1A")  # pylint: disable=W1401
+            else:
+                self._progress_started = True
 
-        # This is a little trick we do in order to make sure that the next
-        # message will always start from a new line - we switch to the new
-        # line after each progress update and move the cursor up. As an
-        # example, this is useful when the copying is interrupted by an
-        # exception - the error message will start form new line.
-        if self._progress_started:
-            # The "move cursor up" escape sequence
-            self._progress_file.write('\033[1A')  # pylint: disable=W1401
-        else:
-            self._progress_started = True
+            self._progress_file.write(progress)
+            self._progress_file.flush()
 
-        self._progress_file.write(progress)
-        self._progress_file.flush()
+        # Update psplash progress when configured. This is using a best effort
+        # strategy to not affect the writing process when psplash breaks, is
+        # not available early enough or screen is not available.
+        if self._psplash_pipe and self.mapped_cnt:
+            try:
+                mode = os.O_WRONLY | os.O_NONBLOCK
+                with os.fdopen(os.open(self._psplash_pipe, mode), "w") as p_fo:
+                    p_fo.write("PROGRESS %d\n" % percent)
+            except:
+                pass
 
     def _get_block_ranges(self):
         """
@@ -441,7 +572,7 @@ class BmapCopy(object):
             # The range of blocks has the "X - Y" format, or it can be just "X"
             # in old bmap format versions. First, split the blocks range string
             # and strip white-spaces.
-            split = [x.strip() for x in blocks_range.split('-', 1)]
+            split = [x.strip() for x in blocks_range.split("-", 1)]
 
             first = int(split[0])
             if len(split) > 1:
@@ -504,13 +635,15 @@ class BmapCopy(object):
                     try:
                         buf = self._f_image.read(length * self.block_size)
                     except IOError as err:
-                        raise Error("error while reading blocks %d-%d of the "
-                                    "image file '%s': %s"
-                                    % (start, end, self._image_path, err))
+                        raise Error(
+                            "error while reading blocks %d-%d of the "
+                            "image file '%s': %s" % (start, end, self._image_path, err)
+                        )
 
                     if not buf:
-                        _log.debug("no more data to read from file '%s'",
-                                   self._image_path)
+                        _log.debug(
+                            "no more data to read from file '%s'", self._image_path
+                        )
                         self._batch_queue.put(None)
                         return
 
@@ -518,17 +651,19 @@ class BmapCopy(object):
                         hash_obj.update(buf)
 
                     blocks = (len(buf) + self.block_size - 1) // self.block_size
-                    _log.debug("queueing %d blocks, queue length is %d" %
-                               (blocks, self._batch_queue.qsize()))
+                    _log.debug(
+                        "queueing %d blocks, queue length is %d"
+                        % (blocks, self._batch_queue.qsize())
+                    )
 
-                    self._batch_queue.put(("range", start, start + blocks - 1,
-                                           buf))
+                    self._batch_queue.put(("range", start, start + blocks - 1, buf))
 
                 if verify and chksum and hash_obj.hexdigest() != chksum:
-                    raise Error("checksum mismatch for blocks range %d-%d: "
-                                "calculated %s, should be %s (image file %s)"
-                                % (first, last, hash_obj.hexdigest(),
-                                   chksum, self._image_path))
+                    raise Error(
+                        "checksum mismatch for blocks range %d-%d: "
+                        "calculated %s, should be %s (image file %s)"
+                        % (first, last, hash_obj.hexdigest(), chksum, self._image_path)
+                    )
         # Silence pylint warning about catching too general exception
         # pylint: disable=W0703
         except Exception:
@@ -550,7 +685,7 @@ class BmapCopy(object):
         # Create the queue for block batches and start the reader thread, which
         # will read the image in batches and put the results to '_batch_queue'.
         self._batch_queue = Queue.Queue(self._batch_queue_len)
-        thread.start_new_thread(self._get_data, (verify, ))
+        thread.start_new_thread(self._get_data, (verify,))
 
         blocks_written = 0
         bytes_written = 0
@@ -566,8 +701,7 @@ class BmapCopy(object):
             try:
                 os.ftruncate(self._f_dest.fileno(), self.image_size)
             except OSError as err:
-                raise Error("cannot truncate file '%s': %s"
-                            % (self._dest_path, err))
+                raise Error("cannot truncate file '%s': %s" % (self._dest_path, err))
 
         # Read the image in '_batch_blocks' chunks and write them to the
         # destination file
@@ -598,11 +732,13 @@ class BmapCopy(object):
             try:
                 self._f_dest.write(buf)
             except IOError as err:
-                raise Error("error while writing blocks %d-%d of '%s': %s"
-                            % (start, end, self._dest_path, err))
+                raise Error(
+                    "error while writing blocks %d-%d of '%s': %s"
+                    % (start, end, self._dest_path, err)
+                )
 
             self._batch_queue.task_done()
-            blocks_written += (end - start + 1)
+            blocks_written += end - start + 1
             bytes_written += len(buf)
 
             self._update_progress(blocks_written)
@@ -614,19 +750,25 @@ class BmapCopy(object):
         # This is just a sanity check - we should have written exactly
         # 'mapped_cnt' blocks.
         if blocks_written != self.mapped_cnt:
-            raise Error("wrote %u blocks from image '%s' to '%s', but should "
-                        "have %u - bmap file '%s' does not belong to this "
-                        "image"
-                        % (blocks_written, self._image_path, self._dest_path,
-                           self.mapped_cnt, self._bmap_path))
+            raise Error(
+                "wrote %u blocks from image '%s' to '%s', but should "
+                "have %u - bmap file '%s' does not belong to this "
+                "image"
+                % (
+                    blocks_written,
+                    self._image_path,
+                    self._dest_path,
+                    self.mapped_cnt,
+                    self._bmap_path,
+                )
+            )
 
         if self._dest_is_regfile:
             # Make sure the destination file has the same size as the image
             try:
                 os.ftruncate(self._f_dest.fileno(), self.image_size)
             except OSError as err:
-                raise Error("cannot truncate file '%s': %s"
-                            % (self._dest_path, err))
+                raise Error("cannot truncate file '%s': %s" % (self._dest_path, err))
 
         try:
             self._f_dest.flush()
@@ -646,15 +788,16 @@ class BmapCopy(object):
             try:
                 os.fsync(self._f_dest.fileno()),
             except OSError as err:
-                raise Error("cannot synchronize '%s': %s "
-                            % (self._dest_path, err.strerror))
+                raise Error(
+                    "cannot synchronize '%s': %s " % (self._dest_path, err.strerror)
+                )
 
 
 class BmapBdevCopy(BmapCopy):
     """
     This class is a specialized version of 'BmapCopy' which copies the image to
     a block device. Unlike the base 'BmapCopy' class, this class does various
-    optimizations specific to block devices, e.g., switching to the 'noop' I/O
+    optimizations specific to block devices, e.g., switching to the 'none' I/O
     scheduler.
     """
 
@@ -672,8 +815,6 @@ class BmapBdevCopy(BmapCopy):
         self._sysfs_base = None
         self._sysfs_scheduler_path = None
         self._sysfs_max_ratio_path = None
-        self._old_scheduler_value = None
-        self._old_max_ratio_value = None
 
         # If the image size is known, check that it fits the block device
         if self.image_size:
@@ -681,19 +822,29 @@ class BmapBdevCopy(BmapCopy):
                 bdev_size = os.lseek(self._f_dest.fileno(), 0, os.SEEK_END)
                 os.lseek(self._f_dest.fileno(), 0, os.SEEK_SET)
             except OSError as err:
-                raise Error("cannot seed block device '%s': %s "
-                            % (self._dest_path, err.strerror))
+                raise Error(
+                    "cannot seed block device '%s': %s "
+                    % (self._dest_path, err.strerror)
+                )
 
             if bdev_size < self.image_size:
-                raise Error("the image file '%s' has size %s and it will not "
-                            "fit the block device '%s' which has %s capacity"
-                            % (self._image_path, self.image_size_human,
-                               self._dest_path, human_size(bdev_size)))
+                raise Error(
+                    "the image file '%s' has size %s and it will not "
+                    "fit the block device '%s' which has %s capacity"
+                    % (
+                        self._image_path,
+                        self.image_size_human,
+                        self._dest_path,
+                        human_size(bdev_size),
+                    )
+                )
 
         # Construct the path to the sysfs directory of our block device
         st_rdev = os.fstat(self._f_dest.fileno()).st_rdev
-        self._sysfs_base = "/sys/dev/block/%s:%s/" % \
-                           (os.major(st_rdev), os.minor(st_rdev))
+        self._sysfs_base = "/sys/dev/block/%s:%s/" % (
+            os.major(st_rdev),
+            os.minor(st_rdev),
+        )
 
         # Check if the 'queue' sub-directory exists. If yes, then our block
         # device is entire disk. Otherwise, it is a partition, in which case we
@@ -704,72 +855,6 @@ class BmapBdevCopy(BmapCopy):
         self._sysfs_scheduler_path = self._sysfs_base + "queue/scheduler"
         self._sysfs_max_ratio_path = self._sysfs_base + "bdi/max_ratio"
 
-    def _tune_block_device(self):
-        """
-        Tune the block device for better performance:
-        1. Switch to the 'noop' I/O scheduler if it is available - sequential
-           write to the block device becomes a lot faster comparing to CFQ.
-        2. Limit the write buffering - we do not need the kernel to buffer a
-           lot of the data we send to the block device, because we write
-           sequentially. Limit the buffering.
-
-        The old settings are saved in order to be able to restore them later.
-        """
-        # Switch to the 'noop' I/O scheduler
-        try:
-            with open(self._sysfs_scheduler_path, "r+") as f_scheduler:
-                contents = f_scheduler.read()
-                f_scheduler.seek(0)
-                f_scheduler.write("noop")
-        except IOError as err:
-            _log.debug("failed to enable I/O optimization, expect "
-                         "suboptimal speed (reason: cannot switch to the "
-                         "'noop' I/O scheduler: %s or blk-mq in use)" % err)
-        else:
-            # The file contains a list of schedulers with the current
-            # scheduler in square brackets, e.g., "noop deadline [cfq]".
-            # Fetch the name of the current scheduler.
-            import re
-
-            match = re.match(r'.*\[(.+)\].*', contents)
-            if match:
-                self._old_scheduler_value = match.group(1)
-
-        # Limit the write buffering, because we do not need too much of it when
-        # writing sequntially. Excessive buffering makes some systems not very
-        # responsive, e.g., this was observed in Fedora 17.
-        try:
-            with open(self._sysfs_max_ratio_path, "r+") as f_ratio:
-                self._old_max_ratio_value = f_ratio.read()
-                f_ratio.seek(0)
-                f_ratio.write("1")
-        except IOError as err:
-            _log.warning("failed to disable excessive buffering, expect "
-                         "worse system responsiveness (reason: cannot set "
-                         "max. I/O ratio to 1: %s)" % err)
-
-    def _restore_bdev_settings(self):
-        """
-        Restore old block device settings which we changed in
-        '_tune_block_device()'.
-        """
-
-        if self._old_scheduler_value is not None:
-            try:
-                with open(self._sysfs_scheduler_path, "w") as f_scheduler:
-                    f_scheduler.write(self._old_scheduler_value)
-            except IOError as err:
-                raise Error("cannot restore the '%s' I/O scheduler: %s"
-                            % (self._old_scheduler_value, err))
-
-        if self._old_max_ratio_value is not None:
-            try:
-                with open(self._sysfs_max_ratio_path, "w") as f_ratio:
-                    f_ratio.write(self._old_max_ratio_value)
-            except IOError as err:
-                raise Error("cannot set the max. I/O ratio back to '%s': %s"
-                            % (self._old_max_ratio_value, err))
-
     def copy(self, sync=True, verify=True):
         """
         The same as in the base class but tunes the block device for better
@@ -784,11 +869,51 @@ class BmapBdevCopy(BmapCopy):
         synchronizing from time to time.
         """
 
-        self._tune_block_device()
+        # Tune the block device for better performance:
+        # 1. Switch to the 'none' (the successor of 'noop' since the switch to
+        #    multiqueue schedulers) I/O scheduler if it is available - sequential
+        #    write to the block device becomes a lot faster comparing to CFQ.
+        # 2. Limit the write buffering - we do not need the kernel to buffer a lot of
+        #    the data we send to the block device, because we write sequentially.
+        #    Excessive buffering would make some systems quite unresponsive.
+        #    This was observed e.g. in Fedora 17.
+        # The old settings are saved and restored by the context managers.
 
-        try:
-            BmapCopy.copy(self, sync, verify)
-        except:
-            raise
-        finally:
-            self._restore_bdev_settings()
+        with SysfsChange(self._sysfs_max_ratio_path, "1") as max_ratio_chg, SysfsChange(
+            self._sysfs_scheduler_path, "none"
+        ) as scheduler_chg:
+            if max_ratio_chg.error:
+                _log.warning(
+                    "failed to disable excessive buffering, expect "
+                    "worse system responsiveness (reason: cannot set "
+                    f"max. I/O ratio to 1: {max_ratio_chg.error})"
+                )
+            if scheduler_chg.error:
+                _log.info(
+                    "failed to enable I/O optimization, expect "
+                    "suboptimal speed (reason: cannot switch to the "
+                    f"{max_ratio_chg.temp_value} I/O scheduler: "
+                    f"{max_ratio_chg.old_value or 'unknown scheduler'} in use. "
+                    f"{max_ratio_chg.error})"
+                )
+            if max_ratio_chg.error or scheduler_chg.error:
+                _log.info(
+                    "You may want to set these I/O optimizations through a udev rule "
+                    "like this:\n"
+                    "#/etc/udev/rules.d/60-bmaptool-optimizations.rules\n"
+                    'SUBSYSTEM!="block", GOTO="bmaptool_optimizations_end"\n'
+                    'ACTION!="add|change", GOTO="bmaptool_optimizations_end"\n'
+                    "\n"
+                    'ACTION=="add", SUBSYSTEMS=="usb", ATTRS{idVendor}=="xxxx", '
+                    'ATTRS{idProduct}=="xxxx", TAG+="uaccess"\n'
+                    'SUBSYSTEMS=="usb", ATTRS{idVendor}=="xxxx", '
+                    'ATTRS{idProduct}=="xxxx", ATTR{bdi/min_ratio}="0", '
+                    'ATTR{bdi/max_ratio}="1", ATTR{queue/scheduler}="none"\n'
+                    "\n"
+                    'LABEL="bmaptool_optimizations_end"\n'
+                    "\n"
+                    "For attributes to match, try\n"
+                    f"udevadm info -a {self._dest_path}"
+                )
+
+            super().copy(sync, verify)
