@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# vim: ts=4 sw=4 et ai si
+# vim: ts=4 sw=4 tw=88 et ai si
 #
 # Copyright (c) 2012-2014 Intel, Inc.
 # License: GPLv2
@@ -18,10 +18,11 @@
 This module allows opening and reading local and remote files and decompress
 them on-the-fly if needed. Remote files are read using urllib (except of
 "ssh://" URLs, which are handled differently). Supported file extentions are:
-'bz2', 'gz', 'xz', 'lzo' and a "tar" version of them: 'tar.bz2', 'tbz2', 'tbz',
-'tb2', 'tar.gz', 'tgz', 'tar.xz', 'txz', 'tar.lzo', 'tzo', 'tar.lz4', 'tlz4'.
+'bz2', 'gz', 'xz', 'lzo', 'zst' and a "tar" version of them: 'tar.bz2', 'tbz2',
+'tbz', 'tb2', 'tar.gz', 'tgz', 'tar.xz', 'txz', 'tar.lzo', 'tzo', 'tar.lz4',
+'tlz4', '.tar.zst', 'tzst'.
 This module uses the following system programs for decompressing: pbzip2, bzip2,
-gzip, pigz, xz, lzop, lz4, tar and unzip.
+gzip, pigz, xz, lzop, lz4, zstd, tar and unzip.
 """
 
 import os
@@ -31,6 +32,7 @@ import sys
 import logging
 import threading
 import subprocess
+import netrc
 from six.moves.urllib import parse as urlparse
 from bmaptools import BmapHelpers
 
@@ -51,9 +53,21 @@ _log = logging.getLogger(__name__)  # pylint: disable=C0103
 # pylint: disable=R0915
 
 # A list of supported compression types
-SUPPORTED_COMPRESSION_TYPES = ('bz2', 'gz', 'xz', 'lzo', 'lz4', 'tar.gz',
-                               'tar.bz2', 'tar.xz', 'tar.lzo', 'tar.lz4',
-                               'zip')
+SUPPORTED_COMPRESSION_TYPES = (
+    "bz2",
+    "gz",
+    "xz",
+    "lzo",
+    "lz4",
+    "zst",
+    "tar.gz",
+    "tar.bz2",
+    "tar.xz",
+    "tar.lzo",
+    "tar.lz4",
+    "tar.zst",
+    "zip",
+)
 
 
 def _fake_seek_forward(file_obj, cur_pos, offset, whence=os.SEEK_SET):
@@ -68,14 +82,16 @@ def _fake_seek_forward(file_obj, cur_pos, offset, whence=os.SEEK_SET):
     elif whence == os.SEEK_CUR:
         new_pos = cur_pos + offset
     else:
-        raise Error("'seek()' method requires the 'whence' argument "
-                    "to be %d or %d, but %d was passed"
-                    % (os.SEEK_SET, os.SEEK_CUR, whence))
+        raise Error(
+            "'seek()' method requires the 'whence' argument "
+            "to be %d or %d, but %d was passed" % (os.SEEK_SET, os.SEEK_CUR, whence)
+        )
 
     if new_pos < cur_pos:
-        raise Error("''seek()' method supports only seeking forward, "
-                    "seeking from %d to %d is not allowed"
-                    % (cur_pos, new_pos))
+        raise Error(
+            "''seek()' method supports only seeking forward, "
+            "seeking from %d to %d is not allowed" % (cur_pos, new_pos)
+        )
 
     length = new_pos - cur_pos
     to_read = length
@@ -87,8 +103,7 @@ def _fake_seek_forward(file_obj, cur_pos, offset, whence=os.SEEK_SET):
         to_read -= len(buf)
 
     if to_read < 0:
-        raise Error("seeked too far: %d instead of %d"
-                    % (new_pos - to_read, new_pos))
+        raise Error("seeked too far: %d instead of %d" % (new_pos - to_read, new_pos))
 
     return new_pos - to_read
 
@@ -99,6 +114,7 @@ class Error(Exception):
     one type of exceptions, and we basically throw human-readable problem
     description in case of errors.
     """
+
     pass
 
 
@@ -119,8 +135,10 @@ def _decode_sshpass_exit_code(code):
     elif code == 5:
         result = "invalid/incorrect password"
     elif code == 6:
-        result = "host public key is unknown. sshpass exits without " \
-                 "confirming the new key"
+        result = (
+            "host public key is unknown. sshpass exits without "
+            "confirming the new key"
+        )
     elif code == 255:
         # SSH result =s 255 on any error
         result = "ssh error"
@@ -147,7 +165,7 @@ class TransRead(object):
         # unknown
         self.size = None
         # Type of the compression of the file
-        self.compression_type = 'none'
+        self.compression_type = "none"
         # Whether the 'bz2file' PyPI module was found
         self.bz2file_found = False
         # Whether the file is behind an URL
@@ -173,7 +191,10 @@ class TransRead(object):
         self._pos = 0
 
         try:
-            self._f_objs.append(open(self.name, "rb"))
+            if self.name == "-":
+                self._f_objs.append(sys.stdin.buffer)
+            else:
+                self._f_objs.append(open(self.name, "rb"))
         except IOError as err:
             if err.errno == errno.ENOENT:
                 # This is probably an URL
@@ -187,14 +208,21 @@ class TransRead(object):
         """The class destructor which closes opened files."""
         self._done = True
 
-        for child in self._child_processes:
-            child.kill()
+        if getattr(self, "_f_objs"):
+            for file_obj in self._f_objs:
+                file_obj.close()
+            self._f_objs = None
 
-        if self._rthread:
+        if getattr(self, "_rthread"):
             self._rthread.join()
+            self._rthread = None
 
-        for file_obj in self._f_objs:
-            file_obj.close()
+        if getattr(self, "_child_processes"):
+            for child in self._child_processes:
+                if child.poll() is None:
+                    child.kill()
+                    child.wait()
+            self._child_processes = []
 
     def _read_thread(self, f_from, f_to):
         """
@@ -205,16 +233,17 @@ class TransRead(object):
         """
 
         chunk_size = 1024 * 1024
-        while not self._done:
-            buf = f_from.read(chunk_size)
-            if not buf:
-                break
+        try:
+            while not self._done:
+                buf = f_from.read(chunk_size)
+                if not buf:
+                    break
 
-            f_to.write(buf)
-
-        # This will make sure the process decompressor gets EOF and exits, as
-        # well as ublocks processes waiting on decompressor's stdin.
-        f_to.close()
+                f_to.write(buf)
+        finally:
+            # This will make sure the process decompressor gets EOF and exits, as
+            # well as ublocks processes waiting on decompressor's stdin.
+            f_to.close()
 
     def _open_compressed_file(self):
         """
@@ -225,32 +254,39 @@ class TransRead(object):
 
         def is_gzip(name):
             """Returns 'True' if file 'name' is compressed with 'gzip'."""
-            if name.endswith('.gzip') or \
-               (name.endswith('.gz') and not name.endswith('.tar.gz')):
+            if name.endswith(".gzip") or (
+                name.endswith(".gz") and not name.endswith(".tar.gz")
+            ):
                 return True
             return False
 
         def is_bzip2(name):
             """Returns 'True' if file 'name' is compressed with 'bzip2'."""
-            if name.endswith('.bz2') and not name.endswith('.tar.bz2'):
+            if name.endswith(".bz2") and not name.endswith(".tar.bz2"):
                 return True
             return False
 
         def is_xz(name):
             """Returns 'True' if file 'name' is compressed with 'xz'."""
-            if name.endswith('.xz') and not name.endswith('.tar.xz'):
+            if name.endswith(".xz") and not name.endswith(".tar.xz"):
                 return True
             return False
 
         def is_lzop(name):
             """Returns 'True' if file 'name' is compressed with 'lzop'."""
-            if name.endswith('.lzo') and not name.endswith('.tar.lzo'):
+            if name.endswith(".lzo") and not name.endswith(".tar.lzo"):
                 return True
             return False
 
         def is_lz4(name):
             """Returns 'True' if file 'name' is compressed with 'lz4'."""
-            if name.endswith('.lz4') and not name.endswith('.tar.lz4'):
+            if name.endswith(".lz4") and not name.endswith(".tar.lz4"):
+                return True
+            return False
+
+        def is_zst(name):
+            """Returns 'True' if file 'name' is compressed with 'zstd'."""
+            if name.endswith(".zst") and not name.endswith(".tar.zst"):
                 return True
             return False
 
@@ -260,7 +296,7 @@ class TransRead(object):
             'gzip'.
             """
 
-            if name.endswith('.tar.gz') or name.endswith('.tgz'):
+            if name.endswith(".tar.gz") or name.endswith(".tgz"):
                 return True
             return False
 
@@ -270,8 +306,12 @@ class TransRead(object):
             'bzip2'.
             """
 
-            if name.endswith('.tar.bz2') or name.endswith('.tbz') or \
-               name.endswith('.tbz2') or name.endswith('.tb2'):
+            if (
+                name.endswith(".tar.bz2")
+                or name.endswith(".tbz")
+                or name.endswith(".tbz2")
+                or name.endswith(".tb2")
+            ):
                 return True
             return False
 
@@ -280,7 +320,7 @@ class TransRead(object):
             Returns 'True' if file 'name' is a tar archive compressed with 'xz'.
             """
 
-            if name.endswith('.tar.xz') or name.endswith('.txz'):
+            if name.endswith(".tar.xz") or name.endswith(".txz"):
                 return True
             return False
 
@@ -290,7 +330,7 @@ class TransRead(object):
             'lzop'.
             """
 
-            if name.endswith('.tar.lzo') or name.endswith('.tzo'):
+            if name.endswith(".tar.lzo") or name.endswith(".tzo"):
                 return True
             return False
 
@@ -300,13 +340,23 @@ class TransRead(object):
             'lz4'.
             """
 
-            if name.endswith('.tar.lz4') or name.endswith('.tlz4'):
+            if name.endswith(".tar.lz4") or name.endswith(".tlz4"):
+                return True
+            return False
+
+        def is_tar_zst(name):
+            """
+            Returns 'True' if file 'name' is a tar archive compressed with
+            'zstd'.
+            """
+
+            if name.endswith(".tar.zst") or name.endswith(".tzst"):
                 return True
             return False
 
         archiver = None
         if is_tar_gz(self.name) or is_gzip(self.name):
-            self.compression_type = 'gzip'
+            self.compression_type = "gzip"
             if BmapHelpers.program_is_available("pigz"):
                 decompressor = "pigz"
             else:
@@ -318,7 +368,7 @@ class TransRead(object):
                 archiver = "tar"
                 args = "-x -z -O"
         elif is_tar_bz2(self.name) or is_bzip2(self.name):
-            self.compression_type = 'bzip2'
+            self.compression_type = "bzip2"
             if BmapHelpers.program_is_available("pbzip2"):
                 decompressor = "pbzip2"
             else:
@@ -330,7 +380,7 @@ class TransRead(object):
                 archiver = "tar"
                 args = "-x -j -O"
         elif is_tar_xz(self.name) or is_xz(self.name):
-            self.compression_type = 'xz'
+            self.compression_type = "xz"
             decompressor = "xz"
             if is_xz(self.name):
                 args = "-d -c"
@@ -338,7 +388,7 @@ class TransRead(object):
                 archiver = "tar"
                 args = "-x -J -O"
         elif is_tar_lzo(self.name) or is_lzop(self.name):
-            self.compression_type = 'lzo'
+            self.compression_type = "lzo"
             decompressor = "lzop"
             if is_lzop(self.name):
                 args = "-d -c"
@@ -346,30 +396,46 @@ class TransRead(object):
                 archiver = "tar"
                 args = "-x --lzo -O"
         elif self.name.endswith(".zip"):
-            self.compression_type = 'zip'
+            self.compression_type = "zip"
             decompressor = "funzip"
             args = ""
         elif is_tar_lz4(self.name) or is_lz4(self.name):
-            self.compression_type = 'lz4'
+            self.compression_type = "lz4"
             decompressor = "lz4"
             if is_lz4(self.name):
                 args = "-d -c"
             else:
                 archiver = "tar"
                 args = "-x -Ilz4 -O"
+        elif is_tar_zst(self.name) or is_zst(self.name):
+            self.compression_type = "zst"
+            decompressor = "zstd"
+            if is_zst(self.name):
+                args = "-d"
+            else:
+                archiver = "tar"
+                args = "-x -Izstd -O"
         else:
             if not self.is_url:
                 self.size = os.fstat(self._f_objs[-1].fileno()).st_size
             return
 
+        if archiver == "tar":
+            # This will get rid of messages like:
+            #     tar: Removing leading `/' from member names'.
+            args += " -P -C /"
+
         # Make sure decompressor and the archiver programs are available
         if not BmapHelpers.program_is_available(decompressor):
-            raise Error("the \"%s\" program is not available but it is "
-                        "required decompressing \"%s\""
-                        % (decompressor, self.name))
+            raise Error(
+                'the "%s" program is not available but it is '
+                'required decompressing "%s"' % (decompressor, self.name)
+            )
         if archiver and not BmapHelpers.program_is_available(archiver):
-            raise Error("the \"%s\" program is not available but it is "
-                        "required reading \"%s\"" % (archiver, self.name))
+            raise Error(
+                'the "%s" program is not available but it is '
+                'required reading "%s"' % (archiver, self.name)
+            )
 
         # Start the decompressor process. We'll send the data to its stdin and
         # read the decompressed data from its stdout.
@@ -383,16 +449,21 @@ class TransRead(object):
         else:
             child_stdin = self._f_objs[-1].fileno()
 
-        child_process = subprocess.Popen(args, shell=True,
-                                         bufsize=1024 * 1024,
-                                         stdin=child_stdin,
-                                         stdout=subprocess.PIPE,
-                                         stderr=subprocess.PIPE)
+        child_process = subprocess.Popen(
+            args,
+            shell=True,
+            bufsize=1024 * 1024,
+            stdin=child_stdin,
+            stdout=subprocess.PIPE,
+        )
 
         if child_stdin == subprocess.PIPE:
             # A separate reader thread is created only when we are reading via
             # urllib2.
-            args = (self._f_objs[-1], child_process.stdin, )
+            args = (
+                self._f_objs[-1],
+                child_process.stdin,
+            )
             self._rthread = threading.Thread(target=self._read_thread, args=args)
             self._rthread.daemon = True
             self._rthread.start()
@@ -417,32 +488,44 @@ class TransRead(object):
 
         # Make sure the ssh client program is installed
         if not BmapHelpers.program_is_available("ssh"):
-            raise Error("the \"ssh\" program is not available but it is "
-                        "required for downloading over the ssh protocol")
+            raise Error(
+                'the "ssh" program is not available but it is '
+                "required for downloading over the ssh protocol"
+            )
 
         # Prepare the commands that we are going to run
         if password:
             # In case of password we have to use the sshpass tool to pass the
             # password to the ssh client utility
-            popen_args = ["sshpass",
-                          "-p" + password,
-                          "ssh",
-                          "-o StrictHostKeyChecking=no",
-                          "-o PubkeyAuthentication=no",
-                          "-o PasswordAuthentication=yes",
-                          hostname]
+            popen_args = [
+                "sshpass",
+                "-p" + password,
+                "ssh",
+                "-o StrictHostKeyChecking=no",
+                "-o ClearAllForwardings=yes",
+                "-o ForwardX11=no",
+                "-o PubkeyAuthentication=no",
+                "-o PasswordAuthentication=yes",
+                hostname,
+            ]
 
             # Make sure the sshpass program is installed
             if not BmapHelpers.program_is_available("ssh"):
-                raise Error("the \"sshpass\" program is not available but it "
-                            "is required for password-based SSH authentication")
+                raise Error(
+                    'the "sshpass" program is not available but it '
+                    "is required for password-based SSH authentication"
+                )
         else:
-            popen_args = ["ssh",
-                          "-o StrictHostKeyChecking=no",
-                          "-o PubkeyAuthentication=yes",
-                          "-o PasswordAuthentication=no",
-                          "-o BatchMode=yes",
-                          hostname]
+            popen_args = [
+                "ssh",
+                "-o StrictHostKeyChecking=no",
+                "-o ClearAllForwardings=yes",
+                "-o ForwardX11=no",
+                "-o PubkeyAuthentication=yes",
+                "-o PasswordAuthentication=no",
+                "-o BatchMode=yes",
+                hostname,
+            ]
 
         # Test if we can successfully connect
         child_process = subprocess.Popen(popen_args + ["true"])
@@ -450,24 +533,27 @@ class TransRead(object):
         retcode = child_process.returncode
         if retcode != 0:
             decoded = _decode_sshpass_exit_code(retcode)
-            raise Error("cannot connect to \"%s\": %s (error code %d)"
-                        % (hostname, decoded, retcode))
+            raise Error(
+                'cannot connect to "%s": %s (error code %d)'
+                % (hostname, decoded, retcode)
+            )
 
         # Test if file exists by running "test -f path && test -r path" on the
         # host
         command = "test -f " + path + " && test -r " + path
-        child_process = subprocess.Popen(popen_args + [command],
-                                         bufsize=1024 * 1024,
-                                         stdout=subprocess.PIPE)
+        child_process = subprocess.Popen(popen_args + [command], bufsize=1024 * 1024)
         child_process.wait()
         if child_process.returncode != 0:
-            raise Error("\"%s\" on \"%s\" cannot be read: make sure it "
-                        "exists, is a regular file, and you have read "
-                        "permissions" % (path, hostname))
+            raise Error(
+                '"%s" on "%s" cannot be read: make sure it '
+                "exists, is a regular file, and you have read "
+                "permissions" % (path, hostname)
+            )
 
         # Read the entire file using 'cat'
-        child_process = subprocess.Popen(popen_args + ["cat " + path],
-                                         stdout=subprocess.PIPE)
+        child_process = subprocess.Popen(
+            popen_args + ["cat " + path], stdout=subprocess.PIPE
+        )
 
         # Now the contents of the file should be available from sub-processes
         # stdout
@@ -487,9 +573,10 @@ class TransRead(object):
             This is a small helper function for printing a warning if we cannot
             open the URL for some time.
             """
-            _log.warning("failed to open the URL with %d sec timeout, is the "
-                         "proxy configured correctly? Keep trying ..." %
-                         timeout)
+            _log.warning(
+                "failed to open the URL with %d sec timeout, is the "
+                "proxy configured correctly? Keep trying ..." % timeout
+            )
 
         import socket
 
@@ -507,6 +594,18 @@ class TransRead(object):
         username = parsed_url.username
         password = parsed_url.password
 
+        if not username and not password and parsed_url.scheme in ("http", "https"):
+            try:
+                n = netrc.netrc()
+                auth = n.authenticators(parsed_url.hostname)
+                if auth:
+                    username = auth[0]
+                    password = auth[2]
+            except FileNotFoundError:
+                pass
+            except netc.NetrcParseError as e:
+                _log.error(f"Error parsing line {e.lineno} of {e.filename}: {e.msg}")
+
         if username and password:
             # Unfortunately, in order to handle URLs which contain user name
             # and password (e.g., http://user:password@my.site.org), we need to
@@ -526,7 +625,7 @@ class TransRead(object):
         else:
             opener = urllib.build_opener()
 
-        opener.addheaders = [('User-Agent', 'Mozilla/5.0')]
+        opener.addheaders = [("User-Agent", "Mozilla/5.0")]
         urllib.install_opener(opener)
 
         # Open the URL. First try with a short timeout, and print a message
@@ -548,16 +647,17 @@ class TransRead(object):
                     raise Error("cannot open URL '%s': %s" % (url, err))
             except URLError as err:
                 # Handling the timeout case in Python 2.6
-                if timeout is not None and \
-                   isinstance(err.reason, socket.timeout):
+                if timeout is not None and isinstance(err.reason, socket.timeout):
                     _print_warning(timeout)
                 else:
                     raise Error("cannot open URL '%s': %s" % (url, err))
             except (IOError, ValueError, httplib.InvalidURL) as err:
                 raise Error("cannot open URL '%s': %s" % (url, err))
             except httplib.BadStatusLine:
-                raise Error("cannot open URL '%s': server responds with an "
-                            "HTTP status code that we don't understand" % url)
+                raise Error(
+                    "cannot open URL '%s': server responds with an "
+                    "HTTP status code that we don't understand" % url
+                )
 
         self.is_url = True
         self._f_objs.append(f_obj)
@@ -578,15 +678,15 @@ class TransRead(object):
     def seek(self, offset, whence=os.SEEK_SET):
         """The 'seek()' method, similar to the one file objects have."""
         if self._fake_seek or not hasattr(self._f_objs[-1], "seek"):
-            self._pos = _fake_seek_forward(self._f_objs[-1], self._pos,
-                                           offset, whence)
+            self._pos = _fake_seek_forward(self._f_objs[-1], self._pos, offset, whence)
         else:
             try:
                 self._f_objs[-1].seek(offset, whence)
             except io.UnsupportedOperation:
                 self._fake_seek = True
-                self._pos = _fake_seek_forward(self._f_objs[-1], self._pos,
-                                               offset, whence)
+                self._pos = _fake_seek_forward(
+                    self._f_objs[-1], self._pos, offset, whence
+                )
 
     def tell(self):
         """The 'tell()' method, similar to the one file objects have."""
@@ -605,7 +705,7 @@ class TransRead(object):
         its operations.
         """
 
-        if self.compression_type == 'none' and not self.is_url:
+        if self.compression_type == "none" and not self.is_url:
             return getattr(self._f_objs[-1], name)
         else:
             raise AttributeError
